# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/ddop2',
                'doc_host': 'https://opimwue.github.io',
                'git_url': 'https://github.com/opimwue/ddop2',
                'lib_path': 'ddop2'},
  'syms': { 'ddop2.core': {'ddop2.core.foo': ('core.html#foo', 'ddop2/core.py')},
            'ddop2.dscm.cost': { 'ddop2.dscm.cost.calculate_costs_per_q': ('dscm_cost.html#calculate_costs_per_q', 'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.calculate_marginal_costs_per_q': ( 'dscm_cost.html#calculate_marginal_costs_per_q',
                                                                                     'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.empirical_cdf': ('dscm_cost.html#empirical_cdf', 'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.get_exp_cost': ('dscm_cost.html#get_exp_cost', 'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.get_exp_marginal_cost': ('dscm_cost.html#get_exp_marginal_cost', 'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.get_marginal_cost': ('dscm_cost.html#get_marginal_cost', 'ddop2/dscm/cost.py'),
                                 'ddop2.dscm.cost.get_total_cost': ('dscm_cost.html#get_total_cost', 'ddop2/dscm/cost.py')},
            'ddop2.dscm.curly_brace': { 'ddop2.dscm.curly_brace.curlyBrace': ( 'dscm_curly_brace.html#curlybrace',
                                                                               'ddop2/dscm/curly_brace.py'),
                                        'ddop2.dscm.curly_brace.getAxSize': ( 'dscm_curly_brace.html#getaxsize',
                                                                              'ddop2/dscm/curly_brace.py')},
            'ddop2.dscm.plots': { 'ddop2.dscm.plots.PlotDecisionSliderHist': ( 'dscm_plots.html#plotdecisionsliderhist',
                                                                               'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderHist.__init__': ( 'dscm_plots.html#plotdecisionsliderhist.__init__',
                                                                                        'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderHist.plot_decision_slider_hist_init': ( 'dscm_plots.html#plotdecisionsliderhist.plot_decision_slider_hist_init',
                                                                                                              'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderHist.plot_figure_init': ( 'dscm_plots.html#plotdecisionsliderhist.plot_figure_init',
                                                                                                'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderHist.update_plot_1': ( 'dscm_plots.html#plotdecisionsliderhist.update_plot_1',
                                                                                             'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderHist.update_slider': ( 'dscm_plots.html#plotdecisionsliderhist.update_slider',
                                                                                             'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg': ( 'dscm_plots.html#plotdecisionslidermarg',
                                                                               'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg.__init__': ( 'dscm_plots.html#plotdecisionslidermarg.__init__',
                                                                                        'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg.plot_decision_slider_marg_init': ( 'dscm_plots.html#plotdecisionslidermarg.plot_decision_slider_marg_init',
                                                                                                              'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg.plot_figure_init': ( 'dscm_plots.html#plotdecisionslidermarg.plot_figure_init',
                                                                                                'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg.pre_calc': ( 'dscm_plots.html#plotdecisionslidermarg.pre_calc',
                                                                                        'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.PlotDecisionSliderMarg.update_slider': ( 'dscm_plots.html#plotdecisionslidermarg.update_slider',
                                                                                             'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.interactive_weekday_plot': ( 'dscm_plots.html#interactive_weekday_plot',
                                                                                 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_cost_and_marginal_cost': ( 'dscm_plots.html#plot_cost_and_marginal_cost',
                                                                                    'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_decision_on_time_series': ( 'dscm_plots.html#plot_decision_on_time_series',
                                                                                     'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_demand_per_day': ('dscm_plots.html#plot_demand_per_day', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_demand_per_month': ( 'dscm_plots.html#plot_demand_per_month',
                                                                              'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_hist': ('dscm_plots.html#plot_hist', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_lin_reg': ('dscm_plots.html#plot_lin_reg', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_lines_area': ('dscm_plots.html#plot_lines_area', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_lines_individual': ( 'dscm_plots.html#plot_lines_individual',
                                                                              'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_loss_functions': ('dscm_plots.html#plot_loss_functions', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_lrnw_core': ('dscm_plots.html#plot_lrnw_core', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_normal_dist': ('dscm_plots.html#plot_normal_dist', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_prescriptions_weekday': ( 'dscm_plots.html#plot_prescriptions_weekday',
                                                                                   'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_prescriptions_weekday_order_quantities': ( 'dscm_plots.html#plot_prescriptions_weekday_order_quantities',
                                                                                                    'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_time_series': ('dscm_plots.html#plot_time_series', 'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_time_series_and_hist': ( 'dscm_plots.html#plot_time_series_and_hist',
                                                                                  'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_time_series_base': ( 'dscm_plots.html#plot_time_series_base',
                                                                              'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_time_series_decomp': ( 'dscm_plots.html#plot_time_series_decomp',
                                                                                'ddop2/dscm/plots.py'),
                                  'ddop2.dscm.plots.plot_time_series_split': ( 'dscm_plots.html#plot_time_series_split',
                                                                               'ddop2/dscm/plots.py')},
            'ddop2.utils.validation': {'ddop2.utils.validation.check_cu_co': ('validation.html#check_cu_co', 'ddop2/utils/validation.py')}}}
